@@ -284,3 +284,58 @@ class DelayedImport(object):
     def __setattr__(self, item, value):
         m = DelayedImport._import_now(self)
         return setattr(m, item, value)
+
+
+class DelayedValue(object):
+    """
+    can be used on module-level variables to delay creation
+    """
+    __slots__ = ["builder", "caller"]
+
+    def __init__(self, builder):
+        globals = sys._getframe(1).f_globals
+        caller_name = globals["__name__"]
+        caller = importlib.import_module(caller_name)
+        _set(self, "builder", builder)
+        _set(self, "caller", caller)
+
+    def _build(self):
+        caller = _get(self, "caller")
+        name = ""
+        for n in dir(caller):
+            try:
+                if getattr(caller, n) is self:
+                    name = n
+                    break
+            except Exception:
+                pass
+        else:
+            _error("Can not find variable holding a " + self.__class__.__name__)
+
+        value = _get(self, "builder")()
+        setattr(caller, name, value)
+        return value
+
+    def __call__(self, *args, **kwargs):
+        m = DelayedValue._build(self)
+        return m(*args, **kwargs)
+
+    def __contains__(self, item):
+        m = DelayedValue._build(self)
+        return item in m
+
+    def __getitem__(self, item):
+        m = DelayedValue._build(self)
+        return m[item]
+
+    def __getattribute__(self, item):
+        if item == "__class__":
+            # reflective code, like in unittest, need to know what this is
+            return DelayedImport
+        m = DelayedValue._build(self)
+        return getattr(m, item)
+
+    def __setattr__(self, item, value):
+        m = DelayedValue._build(self)
+        return setattr(m, item, value)
+
